<?php
/*
Plugin Name: WooCommerce Raneri Free Shipping
Plugin URI: http://woothemes.com/woocommerce
Description: Sets free shipping given some conditions
Version: 1.46
Author: Raneri Web Design Snc
Author URI: http://raneri.it
*/
 
/**
 * Check if WooCommerce is active
 */

$woocommerce_active = 1;

if ( ! function_exists( 'is_plugin_active_for_network' ) )
            require_once( ABSPATH . '/wp-admin/includes/plugin.php' );

if ( ! in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) :
    if ( ! is_plugin_active_for_network( 'woocommerce/woocommerce.php' ) ) :
        $woocommerce_active = 0;
    endif;
endif;


if ( $woocommerce_active == 1 ) {
    add_action( 'woocommerce_shipping_init', 'raneri_free_shipping_init' );
    function raneri_free_shipping_init() {
        if ( ! class_exists( 'Raneri_Free_Shipping' ) ) {
            class Raneri_Free_Shipping extends WC_Shipping_Method {
                public function __construct() {
                    $this->id                 = 'raneri_free_shipping'; // Id for your shipping method. Should be uunique.
                    $this->method_title       = __( 'Raneri Free Shipping', 'raneri-free-shipping' );  // Title shown in admin
                    $this->method_description = __( 'Sets free shipping given some conditions', 'raneri-free-shipping' ); // Description shown in admin

                    $this->admin_page_heading   = __( 'Raneri Free Shipping', 'raneri-free-shipping' );
                    $this->admin_page_description   = __( 'Sets free shipping given some conditions', 'raneri-free-shipping' );

                    $this->init();
                }

 
                function init() {
                    load_plugin_textdomain( 'raneri-free-shipping', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/');

                    // Load the settings API
                    $this->init_form_fields(); // This is part of the settings API. Override the method to add your own settings
                    $this->init_settings(); // This is part of the settings API. Loads settings you previously init.
                    $this->enabled              = $this->settings['enabled'];
                    $this->title                = $this->settings['title'];

                    // Save settings in admin if you have any defined
                    add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );

                    // Maybe set motivational phrase
                    add_action('woocommerce_before_cart_table', array($this, 'maybe_motivational_phrase') );
                }


                function init_form_fields() {
                    global $woocommerce;

                    $allowed_countries_default = get_option( 'woocommerce_specific_allowed_countries' );
                    $allowed_countries_choice = $woocommerce->countries->get_allowed_countries();

                    $states_choice = NULL;
                    if( is_array($allowed_countries_default) ){
                        foreach( $allowed_countries_default as $oneCountry ){
                            $states = $woocommerce->countries->get_states($oneCountry);
                            if( is_array($states) && count($states) > 0 ){
                                foreach($states as $state_code=>$state_name){
                                    $states_choice[$state_code] = $state_name; // rawurldecode
                                }
                            }
                        }
                    }


                    $all_shipping_methods = NULL;
                    if( is_admin() && (!isset($GLOBALS['frontend_running']) || $GLOBALS['frontend_running']!='yes') ){
                        $GLOBALS['frontend_running'] = 'yes';
                        $all_shipping_methods_temp = $woocommerce->shipping->load_shipping_methods();
                        $GLOBALS['frontend_running'] = null;

                        foreach( $all_shipping_methods_temp as $key=>$value ){
                            if( $key != 'raneri_free_shipping' )
                                $all_shipping_methods[$key] = $value->method_title;
                        }
                    }


                    $args = array(
                        'type'                     => 'post',
                        'child_of'                 => 0,
                        'parent'                   => '',
                        'orderby'                  => 'name',
                        'order'                    => 'ASC',
                        'hide_empty'               => 1,
                        'hierarchical'             => 1,
                        'exclude'                  => '',
                        'include'                  => '',
                        'number'                   => '',
                        'taxonomy'                 => 'product_cat',
                        'pad_counts'               => false 

                    ); 
                    $categories_arr = get_categories( $args );
                    foreach( $categories_arr as $temp){
                        $categories_choice[ ( rawurldecode($temp->slug) )] = $temp->name;
                    }


                    $this->form_fields = array(
                                'enabled' => array(
                                                                'title'         => __( 'Enable/Disable', 'woocommerce' ),
                                                                'type'          => 'checkbox',
                                                                'label'         => __( 'Enable this shipping method', 'woocommerce' ),
                                                                'default'       => 'no'
                                                        ),
                                'title' => array(
                                                                'title'         => __( 'Method Title', 'woocommerce' ),
                                                                'type'          => 'text',
                                                                'description'   => __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
                                                                'default'       => __( 'Free Shipping', 'woocommerce' )
                                                        ),
                                'welcome_shipping' => array(
                                                                'title'         => __( 'Welcome Shipping', 'raneri-free-shipping' ),
                                                                'type'          => 'checkbox',
                                                                'label'         => __( 'New registered customers always get free shipping', 'raneri-free-shipping' ),
                                                                'description'   => __( "When this option is activated, new customers (registered users with no orders) always get free shipping, even if the conditions below are not met. Users have to register and login first, and then complete the order.", 'raneri-free-shipping' ),
                                                                'default'       => 'no'
                                                        ),
                                'min_amount' => array(
                                                                'title'         => __( 'Minimum amount', 'raneri-free-shipping' ),
                                                                'type'          => 'number',
                                                                'description'   => __( 'Set the minimum total cart amount - excluding shipping costs - to activate free shipping. Zero means no minimum.', 'raneri-free-shipping' ),
                                                                'default'       => 0
                                                        ),
                                'reverse_min_amount' => array(
                                                                'title'         => __( 'Reverse minimum amount?', 'raneri-free-shipping' ),
                                                                'type'          => 'checkbox',
                                                                'label'         => __( 'Make minimum amount a MAXIMUM amount', 'raneri-free-shipping' ),
                                                                'description'   => __( "Check this to make the amount above a MAXIMUM amount: free shipping will be activated until the total amount of the cart is BELOW or equal to that value.", 'raneri-free-shipping' ),
                                                                'default'       => 'no'
                                                        ),
                                'max_weight' => array(
                                                                'title'         => __( 'Maximum weight', 'raneri-free-shipping' ),
                                                                'type'          => 'number',
                                                                'description'   => __( 'Maximum weight in', 'raneri-free-shipping' ) . ' ' . get_option('woocommerce_weight_unit') . '. ' . __( 'Heavier carts will deactivate free shipping. Zero means no maximum weight.', 'raneri-free-shipping' ),
                                                                'default'       => 0
                                                        ),
                                'min_qty' => array(
                                                                'title'         => __( 'Quantity required', 'raneri-free-shipping' ),
                                                                'type'          => 'number',
                                                                'description'   => __( 'How many products in total must be inside the cart to activate free shipping?', 'raneri-free-shipping' ),
                                                                'default'       => 3
                                                        ),
                                'reverse_min_qty' => array(
                                                                'title'         => __( 'Reverse quantity?', 'raneri-free-shipping' ),
                                                                'type'          => 'checkbox',
                                                                'label'         => __( 'Make required quantity a MAXIMUM quantity', 'raneri-free-shipping' ),
                                                                'description'   => __( "Check this to make the quantity above a MAXIMUM quantity. Example: if you enter 3 as quantity and you don't check this, users have to buy at least 3 products to get free shipping; if you CHECK this, free shipping will be available only if users buy 3 OR LESS products", 'raneri-free-shipping' ),
                                                                'default'       => 'no'
                                                        ),
                                'ignore_virtual_products' => array(
                                                                'title'         => __( 'Ignore virtual products?', 'raneri-free-shipping' ),
                                                                'type'          => 'checkbox',
                                                                'label'         => __( 'Exclude virtual and downloadable products from the count of amount/quantity', 'raneri-free-shipping' ),
                                                                'description'   => __( "NOTICE: Downloadable products must be marked ALSO as virtual to be excluded from the count", 'raneri-free-shipping' ),
                                                                'default'       => 'yes'
                                                        ),
                                'motivational_phrase' => array(
                                                                'title'         => __( 'Motivational Phrase', 'raneri-free-shipping' ),
                                                                'type'          => 'text',
                                                                'description'   => __( 'Sets a phrase to encourage users to reach the minimum amount to get free shipping. (it will appear above the cart -- set empty to disable -- works only with minimum amount and NO minimum quantity)', 'raneri-free-shipping' ),
                                                                'default'       => __( 'Add %s of products to get free shipping!', 'raneri-free-shipping' )
                                                        ),
                                'motivational_phrase_background' => array(
                                                                'title'         => __( 'Motivational Phrase Background', 'raneri-free-shipping' ),
                                                                'type'          => 'text',
                                                                'description'   => __( 'Choose a color for the background of the motivational phrase.', 'raneri-free-shipping' ) . ' ' . __( 'Hexadecimal format, you can generate color codes here:', 'raneri-free-shipping' ) . ' <a href="http://www.w3schools.com/tags/ref_colorpicker.asp" target="_blank">HTML Color Picker</a>',
                                                                'default'       => '#FFFF00'
                                                        ),
                                'categories' => array(
                                                                'title'         => __( 'Categories', 'woocommerce' ),
                                                                'type'          => 'multiselect',
                                                                'css'       => 'width: 450px;',
                                                                'class'     => 'chosen_select',
                                                                'default'   => '',
                                                                'description'   => __( 'Limit free shipping to products in the selected categories (if the cart contains products from other categories, free shipping will be deactivated even if it includes products from the categories selected here). Leave blank to allow free shipping for every category.', 'raneri-free-shipping' ),
                                                                'options'   => $categories_choice
                                                        ),
                                'categories_always_free' => array(
                                                                'title'         => __( 'These categories always trigger free shipping', 'raneri-free-shipping' ),
                                                                'type'          => 'checkbox',
                                                                'label'         => __( 'When this option is activated free shipping becomes available as soon as one product from the categories above is added to the cart, EVEN IF there are also products from other categories', 'raneri-free-shipping' ),
                                                                'default'       => 'no'
                                                        ),
                                'countries' => array(
                                                                'title'         => __( 'Countries', 'woocommerce' ),
                                                                'type'          => 'multiselect',
                                                                'css'       => 'width: 450px;',
                                                                'class'     => 'chosen_select',
                                                                'default'   => $allowed_countries_default,
                                                                'description'   => __( 'Countries for which this shipping method is available', 'raneri-free-shipping' ),
                                                                'options'   => $allowed_countries_choice
                                                        ),
                                'countries_all' => array(
                                                                'title'         => __( 'All Countries', 'woocommerce' ),
                                                                'type'          => 'checkbox',
                                                                'label'         => __( 'When this option is activated, free shipping is available for every country (the setting above is ignored).', 'raneri-free-shipping' ),
                                                                'default'       => 'no'
                                                        ),
                                'exclude_states' => array(
                                                                'title'         => __( 'Exclude states/provinces', 'raneri-free-shipping' ),
                                                                'type'          => 'multiselect',
                                                                'css'       => 'width: 450px;',
                                                                'class'     => 'chosen_select',
                                                                'default'   => '',
                                                                'description'   => __( "Enter the States or Provinces for which this shipping method is NOT available (e.g.: insert Alaska to exclude it from free shipping, even if United States is included in the Countries list above).", 'raneri-free-shipping') . "<br />" . __("NOTICE: If you can't find the states or the provinces of the country of your interest, you must add it to the list of available countries into the General WooCommerce settings, then come back here. If they continue to be missing, probably they aren't available by default. For example WooCommerce doesn't include French Departments; see", 'raneri-free-shipping' ) . ' <a href="http://docs.woothemes.com/document/addmodify-states/">' . __("the official WooCommerce docs", 'raneri-free-shipping') . '</a> ' . __("to add custom states or provinces.", 'raneri-free-shipping'),
                                                                'options'   => $states_choice
                                                        ),
                                'dont_remove_shipping_methods' => array(
                                                                'title'         => __( "Don't remove these shipping methods", 'raneri-free-shipping' ),
                                                                'type'          => 'multiselect',
                                                                'css'       => 'width: 450px;',
                                                                'class'     => 'chosen_select',
                                                                'default'   => '',
                                                                'description'   => __( 'Choose one or more shipping methods that must NOT be removed even if free shipping is available (useful for local pickup options, etc.)', 'raneri-free-shipping' ),
                                                                'options'   => $all_shipping_methods
                                                        ),
                                'disallow_free_if_costs_more_than' => array(
                                                                'title'         => __( 'Disallow free shipping for expansive deliveries', 'raneri-free-shipping' ),
                                                                'type'          => 'number',
                                                                'description'   => __( 'Deactivate free shipping when the BEST shipping rate for the order is over the value you put here. Leave zero to skip this check.', 'raneri-free-shipping' ),
                                                                'default'       => 0
                                                        ),
                                );
                }


                public function validate_countries_field($key)
                {
                        $value = $_POST[ $this->plugin_id . $this->id . '_' . $key ];

                        if ( !is_array( $value ) || count($value)<1 ){
                            die( '<script>alert(\'' . __( 'You must specify at least one country for which free shipping is available.', 'raneri-free-shipping' ) . '\'); history.back(-1);</script>'  );
                        }

                        return $value;
                }


                /*
                public function validate_countries_field($key)
                {
                        $value = $_POST[ $this->plugin_id . $this->id . '_' . $key ];

                        if ( !is_array( $value ) || count($value)<1 ){
                            $this->errors[] = $key;
                        }

                        return $value;
                }


                public function display_errors( ) {
                        error_log('displaying');
                        // loop through each error and display it
                        foreach ( $this->errors as $key => $value ) {
                            ?>
                            <div class="error">
                                <p><?php echo __( 'You must specify at least one country for which free shipping is available.', 'raneri-free-shipping' ); ?></p>
                            </div>
                            <?php
                        }
                }
                */
 

                public function calculate_shipping( $package ) {
                    global $woocommerce;

                    $category_filter = 0;
                    $is_free_category_present = 0;

                    if( $this->settings['categories']!=NULL ){
                        foreach( $woocommerce->cart->cart_contents as $oneProduct ){
                            $product_cats = wp_get_post_terms( $oneProduct['product_id'], 'product_cat' );
                            foreach( $product_cats as $oneProductCat ){
                                $category_filter = 1;
                                if( in_array( rawurldecode($oneProductCat->slug), $this->settings['categories'] ) ){
                                    $category_filter = 0;
                                    $is_free_category_present = 1;
                                    break;
                                }
                            }

                            if( ($this->settings['categories_always_free']!='yes' && $category_filter==1) || ($this->settings['categories_always_free']=='yes' && $category_filter == 0) ){
                                break;
                            }
                        }
                    }

                    $cart_contents_values = $this->get_cart_contents_values();
                    $cart_contents_count = $cart_contents_values['cart_contents_count'];
                    $cart_contents_total = $cart_contents_values['cart_contents_total'];

                    if(
                        (
                            (
                                $this->settings['reverse_min_amount'] != 'yes' && $cart_contents_total >= $this->settings['min_amount']
                                ||
                                $this->settings['reverse_min_amount'] == 'yes' && $cart_contents_total <= $this->settings['min_amount']
                            )
                            &&
                            (
                                $this->settings['reverse_min_qty'] != 'yes' && $cart_contents_count >= $this->settings['min_qty']
                                ||
                                $this->settings['reverse_min_qty'] == 'yes' && $cart_contents_count <= $this->settings['min_qty']
                            )
                        ) &&
                        (
                            (
                                !isset( $this->settings['exclude_states'] ) ||
                                !is_array( $this->settings['exclude_states'] ) || (
                                                                                        is_array( $this->settings['exclude_states'] ) &&
                                                                                        !in_array($package['destination']['state'], $this->settings['exclude_states'])
                                                                                  )
                            )
                            &&
                            (
                                $this->settings['countries_all'] == 'yes' ||
                                in_array($package['destination']['country'], $this->settings['countries'])
                            )
                            &&
                            (
                                $this->settings['max_weight'] == 0 ||
                                $woocommerce->cart->cart_contents_weight <= $this->settings['max_weight']
                            )
                            &&
                            (
                                $category_filter == 0 || ( $this->settings['categories_always_free']=='yes' && $is_free_category_present==1)
                            )
                        )
                    ){

                        $rate = array(
                            'id' => $this->id,
                            'label' => $this->title,
                            'cost' => 0.00,
                            'calc_tax' => 'per_item'
                        );
                        // Register the rate
                        $this->add_rate( $rate );
                    }
                }


                function get_cart_contents_values(){
                    global $woocommerce;

                    if( $this->settings['ignore_virtual_products'] == 'no' ){
                        $cart_contents_count = $woocommerce->cart->cart_contents_count;
                        $cart_contents_total = $woocommerce->cart->cart_contents_total;
                    }
                    else{
                        $cart_contents_count = 0;
                        $cart_contents_total = 0;

                        $items = $woocommerce->cart->get_cart();
                        foreach ( $woocommerce->cart->get_cart() as $cart_item_key => $values ) {
                            $_product = $values['data'];

                            if( $_product->needs_shipping() ){
                                $cart_contents_count += $values['quantity'];
                                $cart_contents_total += $_product->price * $values['quantity'];
                            }
                        }
                    }

                    return array(
                        'cart_contents_count' => $cart_contents_count,
                        'cart_contents_total' => $cart_contents_total
                    );
                }


                function maybe_motivational_phrase(){
                    global $woocommerce;
                    $settings = get_option( 'woocommerce_raneri_free_shipping_settings' );

                    $cart_contents_values = $this->get_cart_contents_values();
                    $cart_contents_total = $cart_contents_values['cart_contents_total'];

                    if( $settings['min_qty']==0 && $this->settings['reverse_min_amount'] != 'yes' && trim($settings['motivational_phrase'])!='' && $cart_contents_total < $settings['min_amount'] ){
                        echo sprintf( '<strong class="raneri_free_shipping_motivational_phrase" style="background:' .$settings['motivational_phrase_background']. '">' .$settings['motivational_phrase']. "</strong>", get_woocommerce_currency_symbol() . " " . number_format($settings['min_amount']-$cart_contents_total, 2) );
                    }
                }
            }
        }
    }

 
    add_filter( 'woocommerce_shipping_methods', 'add_your_shipping_method' );
    function add_your_shipping_method( $methods ) {
        $methods[] = 'raneri_free_shipping';
        return $methods;
    }


    // Hide ALL shipping options when free shipping is available
    add_filter( 'woocommerce_package_rates', 'hide_all_shipping_when_free_is_available_after_2_1' , 10, 1 );
    function hide_all_shipping_when_free_is_available_after_2_1( $rates ) {
        
        if ( isset($rates['raneri_free_shipping']) ) {
            // STEP 1: find the best from other shipping options rates
            $settings = get_option('woocommerce_raneri_free_shipping_settings');
            if( isset($settings['disallow_free_if_costs_more_than']) && $settings['disallow_free_if_costs_more_than']>0 ){
                $other_rates = array();

                foreach( $rates as $rate ){
                    if( $rate->id!='raneri_free_shipping' ){
                        $other_rates[] = $rate->cost;
                    }
                }

                if( count($other_rates)>0 ){
                    $best_rate = min($other_rates);
                }
                else{
                    $best_rate = 0;
                }

                if( $best_rate > $settings['disallow_free_if_costs_more_than'] ){
                    unset( $rates['raneri_free_shipping'] );
                    return $rates;
                }
            }


            // STEP 2: Hide other shipping options (but don't hide the forced ones)
            // Get Free Shipping array into a new array
            $freeshipping = array();
            $freeshipping = $rates['raneri_free_shipping'];

            // if there is one or more forced shipping, we save them
            $settings = get_option('woocommerce_raneri_free_shipping_settings');

            $available_methods_remaining = null;

            if( isset( $settings['dont_remove_shipping_methods']) && is_array($settings['dont_remove_shipping_methods']) ){
                foreach($rates as $temp){
                    if( in_array($temp->method_id, $settings['dont_remove_shipping_methods']) )
                        $available_methods_remaining[$temp->id] = $temp;
                }
            }
     
            $rates = $available_methods_remaining;
            $rates['raneri_free_shipping'] = $freeshipping;
        }
         
        return $rates;
    }

}